"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    comparePredicates: ()=>comparePredicates,
    predicateIntersection: ()=>predicateIntersection,
    predicateUnion: ()=>predicateUnion,
    flattenPredicate: ()=>flattenPredicate,
    isLiteralCondition: ()=>isLiteralCondition,
    resolutionExtendsDomain: ()=>resolutionExtendsDomain
});
const _domainsTs = require("../utils/domains.js");
const _genericsTs = require("../utils/generics.js");
const _branchesTs = require("./branches.js");
const _composeTs = require("./compose.js");
const _discriminateTs = require("./discriminate.js");
const _rulesTs = require("./rules/rules.js");
const emptyRulesIfTrue = (predicate)=>predicate === true ? {} : predicate;
const comparePredicates = (l, r, context)=>{
    if (l === true && r === true) {
        return (0, _composeTs.equality)();
    }
    if (!(0, _domainsTs.hasSubdomain)(l, "Array") && !(0, _domainsTs.hasSubdomain)(r, "Array")) {
        const result = (0, _rulesTs.branchIntersection)(emptyRulesIfTrue(l), emptyRulesIfTrue(r), context);
        return result === l ? l : result === r ? r : result;
    }
    const lBranches = (0, _genericsTs.listFrom)(emptyRulesIfTrue(l));
    const rBranches = (0, _genericsTs.listFrom)(emptyRulesIfTrue(r));
    const comparison = (0, _branchesTs.compareBranches)(lBranches, rBranches, context);
    if (comparison.equalities.length === lBranches.length && comparison.equalities.length === rBranches.length) {
        return (0, _composeTs.equality)();
    }
    if (comparison.lSubconditionsOfR.length + comparison.equalities.length === lBranches.length) {
        return l;
    }
    if (comparison.rSubconditionsOfL.length + comparison.equalities.length === rBranches.length) {
        return r;
    }
    return comparison;
};
const predicateIntersection = (domain, l, r, state)=>{
    state.domain = domain;
    const comparison = comparePredicates(l, r, state);
    if (!(0, _branchesTs.isBranchComparison)(comparison)) {
        return comparison;
    }
    return (0, _genericsTs.collapseIfSingleton)([
        ...comparison.distinctIntersections,
        ...comparison.equalities.map((indices)=>comparison.lConditions[indices[0]]),
        ...comparison.lSubconditionsOfR.map((lIndex)=>comparison.lConditions[lIndex]),
        ...comparison.rSubconditionsOfL.map((rIndex)=>comparison.rConditions[rIndex])
    ]);
};
const predicateUnion = (domain, l, r, type)=>{
    const state = new _composeTs.IntersectionState(type, "|");
    const comparison = comparePredicates(l, r, state);
    if (!(0, _branchesTs.isBranchComparison)(comparison)) {
        return (0, _composeTs.isEquality)(comparison) || comparison === l ? r : comparison === r ? l : // subtype of the other, it consists of two opposite literals
        // and can be simplified to a non-literal boolean.
        domain === "boolean" ? true : [
            emptyRulesIfTrue(l),
            emptyRulesIfTrue(r)
        ];
    }
    return (0, _genericsTs.collapseIfSingleton)([
        ...comparison.lConditions.filter((_, lIndex)=>!comparison.lSubconditionsOfR.includes(lIndex) && !comparison.equalities.some((indexPair)=>indexPair[0] === lIndex)),
        ...comparison.rConditions.filter((_, rIndex)=>!comparison.rSubconditionsOfL.includes(rIndex) && !comparison.equalities.some((indexPair)=>indexPair[1] === rIndex))
    ]);
};
const flattenPredicate = (predicate, context)=>{
    if (predicate === true) {
        return [];
    }
    return (0, _domainsTs.hasSubdomain)(predicate, "Array") ? (0, _discriminateTs.flattenBranches)(predicate, context) : (0, _rulesTs.flattenBranch)(predicate, context);
};
const isLiteralCondition = (predicate)=>typeof predicate === "object" && "value" in predicate;
const resolutionExtendsDomain = (resolution, domain)=>{
    const domains = (0, _genericsTs.keysOf)(resolution);
    return domains.length === 1 && domains[0] === domain;
};
