"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    subdomainIntersection: ()=>subdomainIntersection,
    subdomainPathSegments: ()=>subdomainPathSegments,
    flattenSubdomain: ()=>flattenSubdomain
});
const _errorsTs = require("../../utils/errors.js");
const _composeTs = require("../compose.js");
const _nodeTs = require("../node.js");
const subdomainIntersection = (0, _composeTs.composeIntersection)((l, r, state)=>{
    if (typeof l === "string") {
        if (typeof r === "string") {
            return l === r ? (0, _composeTs.equality)() : state.addDisjoint("subdomain", l, r);
        }
        return l === r[0] ? r : state.addDisjoint("subdomain", l, r[0]);
    }
    if (typeof r === "string") {
        return l[0] === r ? l : state.addDisjoint("subdomain", l[0], r);
    }
    if (l[0] !== r[0]) {
        return state.addDisjoint("subdomain", l[0], r[0]);
    }
    const result = [
        l[0]
    ];
    let lImpliesR = true;
    let rImpliesL = true;
    for(let i = 1; i < l.length; i++){
        const lNode = l[i];
        const rNode = r[i];
        state.path.push(subdomainParameterToPathSegment(l[0], i));
        const parameterResult = (0, _nodeTs.nodeIntersection)(lNode, rNode, state);
        state.path.pop();
        if ((0, _composeTs.isEquality)(parameterResult)) {
            result[i] = lNode;
        } else if (parameterResult === l) {
            result[i] = lNode;
            rImpliesL = false;
        } else if (parameterResult === r) {
            result[i] = rNode;
            lImpliesR = false;
        } else if ((0, _composeTs.isDisjoint)(parameterResult)) {
            return parameterResult;
        } else {
            result[i] = parameterResult;
            lImpliesR = false;
            rImpliesL = false;
        }
    }
    return lImpliesR ? rImpliesL ? (0, _composeTs.equality)() : l : rImpliesL ? r : result;
});
const subdomainParameterToPathSegment = (subdomain, i)=>subdomain === "Array" ? "${number}" : subdomain === "Set" ? "${item}" : subdomain === "Map" ? i === 1 ? "${key}" : "${value}" : (0, _errorsTs.throwInternalError)(`Unexpected parameterized subdomain '${subdomain}'`);
const subdomainPathSegments = {
    "${number}": true,
    "${item}": true,
    "${key}": true,
    "${value}": true
};
const flattenSubdomain = (entries, rule, ctx)=>entries.push([
        "subdomain",
        typeof rule === "string" ? rule : rule[0] === "Map" ? [
            rule[0],
            (0, _nodeTs.flattenNode)(rule[1], ctx),
            (0, _nodeTs.flattenNode)(rule[2], ctx)
        ] : [
            rule[0],
            (0, _nodeTs.flattenNode)(rule[1], ctx)
        ]
    ]);
