"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    hasDomain: ()=>hasDomain,
    domainOf: ()=>domainOf,
    subdomainOf: ()=>subdomainOf,
    hasSubdomain: ()=>hasSubdomain,
    sizeOf: ()=>sizeOf,
    unitsOf: ()=>unitsOf,
    classNameOf: ()=>classNameOf
});
const _genericsTs = require("./generics.js");
const hasDomain = (data, domain)=>domainOf(data) === domain;
const domainOf = (data)=>{
    const builtinType = typeof data;
    return builtinType === "object" ? data === null ? "null" : "object" : builtinType === "function" ? "object" : builtinType;
};
const objectSubdomains = {
    Array,
    Date,
    Error,
    Function,
    Map,
    RegExp,
    Set,
    object: Object
};
const subdomainOf = (data)=>{
    const domain = domainOf(data);
    if (domain !== "object") {
        return domain;
    }
    if (Array.isArray(data)) {
        return "Array";
    }
    // The raw result will be something like [object Date]
    const prototypeName = Object.prototype.toString.call(data).slice(8, -1);
    if ((0, _genericsTs.isKeyOf)(prototypeName, objectSubdomains)) {
        return data instanceof objectSubdomains[prototypeName] ? prototypeName : "object";
    }
    if (prototypeName.endsWith("Error")) {
        return data instanceof Error ? "Error" : "object";
    }
    return "object";
};
const hasSubdomain = (data, subdomain)=>subdomainOf(data) === subdomain;
const sizeOf = (data)=>sizedSubdomains[subdomainOf(data)]?.(data) ?? 0;
const sizedSubdomains = {
    number: (data)=>data,
    string: (data)=>data.length,
    Array: (data)=>data.length
};
const unitsOf = (data)=>sizeUnits[subdomainOf(data)] ?? "";
const sizeUnits = {
    number: "",
    string: "characters",
    Array: "items"
};
const classNameOf = (data)=>Object(data).constructor.name;
