import { hasSubdomain } from "../utils/domains.js";
import { collapseIfSingleton, keysOf, listFrom } from "../utils/generics.js";
import { compareBranches, isBranchComparison } from "./branches.js";
import { equality, IntersectionState, isEquality } from "./compose.js";
import { flattenBranches } from "./discriminate.js";
import { branchIntersection, flattenBranch } from "./rules/rules.js";
const emptyRulesIfTrue = (predicate)=>predicate === true ? {} : predicate;
export const comparePredicates = (l, r, context)=>{
    if (l === true && r === true) {
        return equality();
    }
    if (!hasSubdomain(l, "Array") && !hasSubdomain(r, "Array")) {
        const result = branchIntersection(emptyRulesIfTrue(l), emptyRulesIfTrue(r), context);
        return result === l ? l : result === r ? r : result;
    }
    const lBranches = listFrom(emptyRulesIfTrue(l));
    const rBranches = listFrom(emptyRulesIfTrue(r));
    const comparison = compareBranches(lBranches, rBranches, context);
    if (comparison.equalities.length === lBranches.length && comparison.equalities.length === rBranches.length) {
        return equality();
    }
    if (comparison.lSubconditionsOfR.length + comparison.equalities.length === lBranches.length) {
        return l;
    }
    if (comparison.rSubconditionsOfL.length + comparison.equalities.length === rBranches.length) {
        return r;
    }
    return comparison;
};
export const predicateIntersection = (domain, l, r, state)=>{
    state.domain = domain;
    const comparison = comparePredicates(l, r, state);
    if (!isBranchComparison(comparison)) {
        return comparison;
    }
    return collapseIfSingleton([
        ...comparison.distinctIntersections,
        ...comparison.equalities.map((indices)=>comparison.lConditions[indices[0]]),
        ...comparison.lSubconditionsOfR.map((lIndex)=>comparison.lConditions[lIndex]),
        ...comparison.rSubconditionsOfL.map((rIndex)=>comparison.rConditions[rIndex])
    ]);
};
export const predicateUnion = (domain, l, r, type)=>{
    const state = new IntersectionState(type, "|");
    const comparison = comparePredicates(l, r, state);
    if (!isBranchComparison(comparison)) {
        return isEquality(comparison) || comparison === l ? r : comparison === r ? l : // subtype of the other, it consists of two opposite literals
        // and can be simplified to a non-literal boolean.
        domain === "boolean" ? true : [
            emptyRulesIfTrue(l),
            emptyRulesIfTrue(r)
        ];
    }
    return collapseIfSingleton([
        ...comparison.lConditions.filter((_, lIndex)=>!comparison.lSubconditionsOfR.includes(lIndex) && !comparison.equalities.some((indexPair)=>indexPair[0] === lIndex)),
        ...comparison.rConditions.filter((_, rIndex)=>!comparison.rSubconditionsOfL.includes(rIndex) && !comparison.equalities.some((indexPair)=>indexPair[1] === rIndex))
    ]);
};
export const flattenPredicate = (predicate, context)=>{
    if (predicate === true) {
        return [];
    }
    return hasSubdomain(predicate, "Array") ? flattenBranches(predicate, context) : flattenBranch(predicate, context);
};
export const isLiteralCondition = (predicate)=>typeof predicate === "object" && "value" in predicate;
export const resolutionExtendsDomain = (resolution, domain)=>{
    const domains = keysOf(resolution);
    return domains.length === 1 && domains[0] === domain;
};
