import { isKeyOf } from "./generics.js";
export const hasDomain = (data, domain)=>domainOf(data) === domain;
export const domainOf = (data)=>{
    const builtinType = typeof data;
    return builtinType === "object" ? data === null ? "null" : "object" : builtinType === "function" ? "object" : builtinType;
};
const objectSubdomains = {
    Array,
    Date,
    Error,
    Function,
    Map,
    RegExp,
    Set,
    object: Object
};
export const subdomainOf = (data)=>{
    const domain = domainOf(data);
    if (domain !== "object") {
        return domain;
    }
    if (Array.isArray(data)) {
        return "Array";
    }
    // The raw result will be something like [object Date]
    const prototypeName = Object.prototype.toString.call(data).slice(8, -1);
    if (isKeyOf(prototypeName, objectSubdomains)) {
        return data instanceof objectSubdomains[prototypeName] ? prototypeName : "object";
    }
    if (prototypeName.endsWith("Error")) {
        return data instanceof Error ? "Error" : "object";
    }
    return "object";
};
export const hasSubdomain = (data, subdomain)=>subdomainOf(data) === subdomain;
export const sizeOf = (data)=>sizedSubdomains[subdomainOf(data)]?.(data) ?? 0;
const sizedSubdomains = {
    number: (data)=>data,
    string: (data)=>data.length,
    Array: (data)=>data.length
};
export const unitsOf = (data)=>sizeUnits[subdomainOf(data)] ?? "";
const sizeUnits = {
    number: "",
    string: "characters",
    Array: "items"
};
export const classNameOf = (data)=>Object(data).constructor.name;
