import type { FlatBound } from "../nodes/rules/range.js";
import type { Subdomain } from "../utils/domains.js";
import { domainOf, subdomainOf } from "../utils/domains.js";
import type { constructor, evaluate, evaluateObject, instanceOf } from "../utils/generics.js";
import { Path } from "../utils/paths.js";
import type { ConstrainedRuleTraversalData, TraversalState } from "./traverse.js";
export declare class ArkTypeError extends TypeError {
    cause: Problems;
    constructor(problems: Problems);
}
export declare class Problem {
    code: ProblemCode;
    path: Path;
    private data;
    private rule;
    private writers;
    constructor(code: ProblemCode, path: Path, data: ProblemDataInput, rule: ProblemRuleInput, writers: ProblemWriterConfig<any>);
    toString(): string;
    get message(): string;
    get reason(): string;
    get mustBe(): string;
    get was(): string;
}
declare class ProblemArray extends Array<Problem> {
    private state;
    byPath: Record<string, Problem>;
    constructor(state: TraversalState);
    add<code extends ProblemCode>(code: code, data: ProblemDataInput<code>, input: ProblemRuleInput<code>): void;
    get summary(): string;
    toString(): string;
    throw(): never;
}
export declare const Problems: new (state: TraversalState) => readonly Problem[] & {
    [k in Exclude<keyof ProblemArray, keyof unknown[]>]: ProblemArray[k];
};
export type Problems = instanceOf<typeof Problems>;
export declare const describeSubdomains: (subdomains: Subdomain[]) => string;
export declare const describeBranches: (descriptions: string[]) => string;
/** Each Subdomain's completion for the phrase "Must be _____" */
export declare const subdomainDescriptions: {
    readonly bigint: "a bigint";
    readonly boolean: "boolean";
    readonly null: "null";
    readonly number: "a number";
    readonly object: "an object";
    readonly string: "a string";
    readonly symbol: "a symbol";
    readonly undefined: "undefined";
    readonly Array: "an array";
    readonly Function: "a function";
    readonly Date: "a Date";
    readonly RegExp: "a RegExp";
    readonly Error: "an Error";
    readonly Map: "a Map";
    readonly Set: "a Set";
};
type ProblemRuleInputs = {
    divisor: number;
    class: constructor;
    domain: Subdomain;
    domainBranches: Subdomain[];
    missing: undefined;
    bound: FlatBound;
    regex: RegExp;
    value: unknown;
    valueBranches: unknown[];
    multi: string[];
    branches: Problem[];
};
export type ProblemCode = evaluate<keyof ProblemRuleInputs>;
export type ProblemRuleInput<code extends ProblemCode = ProblemCode> = ProblemRuleInputs[code];
type ProblemDataInputs = {
    [code in ProblemCode]: code extends keyof ConstrainedRuleTraversalData ? ConstrainedRuleTraversalData[code] : unknown;
};
export type ProblemDataInput<code extends ProblemCode = ProblemCode> = ProblemDataInputs[code];
export type ProblemOptions<code extends ProblemCode> = {
    mustBe?: RuleWriter<code>;
    was?: DataWriter<code>;
    reason?: ReasonWriter;
    message?: MessageWriter;
};
export type ProblemWriterConfig<code extends ProblemCode> = Required<ProblemOptions<code>>;
export type RuleWriter<code extends ProblemCode> = (context: ProblemRuleInputs[code]) => string;
export type DataWriter<code extends ProblemCode> = (data: DataWrapper<code extends keyof ConstrainedRuleTraversalData ? ConstrainedRuleTraversalData[code] : unknown>) => string;
export type ReasonWriter = (rule: string, data: string) => string;
export type MessageWriter = (reason: string, path: Path) => string;
export declare class DataWrapper<value = unknown> {
    value: value;
    constructor(value: value);
    toString(): string;
    get domain(): domainOf<value>;
    get subdomain(): subdomainOf<value>;
    get size(): number;
    get units(): "" | NonNullable<{
        string?: string;
        number?: string;
        bigint?: string;
        boolean?: string;
        symbol?: string;
        undefined?: string;
        object?: string;
        null?: string;
        Array?: string;
        Date?: string;
        Error?: string;
        Function?: string;
        Map?: string;
        RegExp?: string;
        Set?: string;
    }[subdomainOf<value>]>;
    get className(): any;
}
export declare const defaultProblemWriters: ProblemsConfig;
export type ProblemsOptions = evaluateObject<{
    all?: ProblemOptions<ProblemCode>;
} & {
    [code in ProblemCode]?: ProblemOptions<code>;
}>;
export type ProblemsConfig = {
    [code in ProblemCode]: ProblemWriterConfig<code>;
};
export declare const compileProblemOptions: (opts: ProblemsOptions | undefined, base?: ProblemsConfig) => ProblemsConfig;
export {};
