import type { Type } from "../main.js";
import type { TraversalEntry, TraversalKey, TraversalNode } from "../nodes/node.js";
import type { SizedData, Subdomain } from "../utils/domains.js";
import type { Dict, extend, List } from "../utils/generics.js";
import { Path } from "../utils/paths.js";
import type { Problem } from "./problems.js";
export declare class TraversalState {
    #private;
    type: Type;
    path: Path;
    problems: readonly Problem[] & {
        byPath: Record<string, Problem>;
        add: <code extends keyof {
            divisor: number;
            class: import("../utils/generics.js").constructor;
            domain: Subdomain;
            domainBranches: Subdomain[];
            missing: undefined;
            bound: import("../nodes/rules/range.js").FlatBound;
            regex: RegExp;
            value: unknown;
            valueBranches: unknown[];
            multi: string[];
            branches: Problem[];
        }>(code: code, data: import("./problems.js").ProblemDataInput<code>, input: import("./problems.js").ProblemRuleInput<code>) => void;
        summary: string;
        throw: () => never;
    };
    failFast: boolean;
    constructor(type: Type);
    traverseResolution(data: unknown, name: string): void;
    traverseBranches(data: unknown, branches: TraversalEntry[][]): void;
}
export declare const traverse: (data: unknown, node: TraversalNode, state: TraversalState) => unknown;
export declare const checkEntries: (data: unknown, entries: List<TraversalEntry>, state: TraversalState) => unknown;
export declare const checkSubdomain: TraversalCheck<"subdomain">;
export type TraversalCheck<k extends TraversalKey> = (data: RuleData<k>, rule: Extract<TraversalEntry, [k, unknown]>[1], state: TraversalState) => void;
export type ConstrainedRuleTraversalData = extend<{
    [k in TraversalKey]?: unknown;
}, {
    regex: string;
    divisor: number;
    bound: SizedData;
    requiredProps: Dict;
    optionalProps: Dict;
    class: object;
}>;
export type RuleData<k extends TraversalKey> = k extends keyof ConstrainedRuleTraversalData ? ConstrainedRuleTraversalData[k] : unknown;
