import type { Dict, evaluate, isTopType, List } from "./generics.js";
export declare const hasDomain: <data, domain extends keyof DomainTypes>(data: data, domain: domain) => data is data & inferDomain<domain>;
type DomainTypes = {
    bigint: bigint;
    boolean: boolean;
    number: number;
    object: object;
    string: string;
    symbol: symbol;
    undefined: undefined;
    null: null;
};
export type inferDomain<domain extends Domain> = Domain extends domain ? unknown : DomainTypes[domain];
export type Domain = evaluate<keyof DomainTypes>;
export type NullishDomain = "undefined" | "null";
export type NonNullishDomain = Exclude<Domain, NullishDomain>;
export type PrimitiveDomain = Exclude<Domain, "object">;
export type Primitive = inferDomain<PrimitiveDomain>;
export type domainOf<data> = isTopType<data> extends true ? Domain : data extends object ? "object" : data extends string ? "string" : data extends number ? "number" : data extends boolean ? "boolean" : data extends undefined ? "undefined" : data extends null ? "null" : data extends bigint ? "bigint" : data extends symbol ? "symbol" : never;
export declare const domainOf: <data>(data: data) => domainOf<data>;
export type ObjectSubdomains = {
    Array: readonly unknown[];
    Date: Date;
    Error: Error;
    Function: Function;
    Map: Map<unknown, unknown>;
    RegExp: RegExp;
    Set: Set<unknown>;
    object: Dict;
};
export type ObjectSubdomain = keyof ObjectSubdomains;
export type Subdomain<domain extends Domain = Domain> = domain | (domain extends "object" ? ObjectSubdomain : never);
export type subdomainOf<data> = isTopType<data> extends true ? Subdomain : data extends object ? object extends data ? ObjectSubdomain : data extends List ? "Array" : data extends Date ? "Date" : data extends Error ? "Error" : data extends Function ? "Function" : data extends Map<unknown, unknown> ? "Map" : data extends RegExp ? "RegExp" : data extends Set<unknown> ? "Set" : "object" : domainOf<data>;
export declare const subdomainOf: <data>(data: data) => subdomainOf<data>;
export type inferSubdomain<subdomain extends Subdomain> = subdomain extends ObjectSubdomain ? ObjectSubdomains[subdomain] : subdomain extends Domain ? DomainTypes[subdomain] : never;
export declare const hasSubdomain: <subdomain extends Subdomain<keyof DomainTypes>>(data: unknown, subdomain: subdomain) => data is inferSubdomain<subdomain>;
export declare const sizeOf: <data>(data: data) => number;
export declare const unitsOf: <data>(data: data) => "" | NonNullable<{
    string?: string;
    number?: string;
    bigint?: string;
    boolean?: string;
    symbol?: string;
    undefined?: string;
    object?: string;
    null?: string;
    Array?: string;
    Date?: string;
    Error?: string;
    Function?: string;
    Map?: string;
    RegExp?: string;
    Set?: string;
}[subdomainOf<data>]>;
export type SizedSubdomain = "number" | "string" | "Array";
export type SizedData = inferSubdomain<SizedSubdomain>;
export declare const classNameOf: <data>(data: data) => any;
export {};
